/**
 * Чтобы использовать этот класс необходимо соблюсти формат входных данных.
 * Необходимо наличие элемента, в котором содержатся карточки. Он должен
 * иметь id, например:
 * <code><div id="unsorted_cards">
 *     <div>карточка 1...</div>
 *     <div>карточку 2</div>
 *     ...
 * </div></code>
 * Названия тега-контейнера и внутренних не важно.
 * На странице должен присутствовать контейнер, в который запишутся
 * отсортированные карточки. Он также должен иметь id, например:
 * <code><ol id="sorted_cards"></ol></code>
 *
 * Чтобы получить список отсортированных карточек, необходимо вызвать
 * метод order:
 * <code>TripSorder.order("unsorted_cards", "sorted_cards");</code>
 * Кроме того, третьим аргументом можно указать название тега, в котором
 * будут содержаться тексты карточек, например результат может быть:
 * <code><ol id="sorted_cards">
 *     <li>card1</li>
 *     <li>card2</li>
 *     ...
 * </ol></code>
 * а может быть:
 * <code><ol id="sorted_cards">
 *     <div>card1</div>
 *     ...
 * </ol></code>
 *
 * Текст каждой карточки должен содержать либо выражение: <code>from
 * [название пункта]</code>
 * и <code>to [название пункта]</code>. Регистр не важен, например:
 * Take train 78A from [Madrid] to [Barcelona]. Seat 45B.
 * Либо выражение: <code><span class="from">название пункта</span></code>
 * и <code><span class="to">название пункта</span></code>, в зависимости от
 * используемой переменной в <code>this.configs.fromToPatterns</code>.
 */
var TripSorter = {

	/**
	 * Различные параметры
	 */
	configs: {
		itemTag: "li",
		fromToPatterns: {
			spanRegex: /<span class=["']from["']>(.*)<\/span>.*?<span class=["']to["']>(.*)<\/span>/i,
			sqBracketsRegex: /from\s*\[([^\]]*)\].*?to\s*\[([^\]]*)\]/i
		}
	},

	/**
	 * В этом поле содержится неотсортированный список карточек, т.е. содержимое
	 * элементов <code><li></code>.
	 */
	stops: {texts: [], orders: []},

	/**
	 * Непосредственно названия остановок
	 */
	points: {to: [], from: [], orders: []},

	/**
	 * Reset-метод, вызывается каждый раз при новом построении, кроме того,
	 * можно вызвать отдельно для сброса внутри класса и очистки конечного списка.
	 *
	 * @param target_id id конечного списка (<code><ol id="target_id"></code>)
	 */
	clear: function(target_id) {
		this.stops = {texts: [], orders: []};
		this.points = {to: [], from: [], orders: []};
		document.getElementById(target_id).innerHTML = '';
	},

	/**
	 * Главный метод. Следует вызывать для получения отсортированного списка
	 * из исходного. Предполагается, что на странице присутствуют оба элемента.
	 *
	 * @param string source_id id исходного списка (<code><ol id="source_id"></code>)
	 * @param string target_id id конечного списка (<code><ol id="target_id"></code>)
	 * @param string tagName тэг, в который заключён текст карточки (по умолчанию
	 * равен <code>this.configs.itemTag</code>)
	 */
	order: function(source_id, target_id, tagName) {
		this.clear(target_id); // reset

		if (tagName !== undefined) {
			this.configs.itemTag = tagName;
		}

		var listops = document.getElementById(source_id).children; // ul#tickets li
		var stops_list = "";
		console.log("* building list...\n\n");
		for (var i = 0; i < listops.length; i++){
			this.stops.texts .push(listops[i].innerHTML); // <li>текст карточки</li>
			this.stops.orders.push(i);
			//var section = listops[i].innerHTML.match(/from\s*\[([^\]]*)\].*?to\s*\[([^\]]*)\]/i);
			var section = listops[i].innerHTML.match(this.configs.fromToPatterns.spanRegex);
			// отдельный регекс: /to\s*\[([^\]]*)\]/i; s.match(...)[1];
			// /from\s*\[([^\]]*)\].*?to\s*\[([^\]]*)\]/i
			if (section) {
				stops_list += "from: " + section[1] + ", to: " + section[2] + "\n";
				this.points['from'].push(section[1]);
				this.points['to'].push(section[2]);
				this.points.orders.push(i);
			}
		}

		console.log(stops_list);
		this.sort();
		this.echoOrderedList(target_id);

		console.log("\n* printing...\n\n");
		for (i in this.points.from) {
			console.log("from: " + this.points.from[i] + ", to: " + this.points.to[i]);
		}
	},

	/**
	 * Сортирует список <code>this.stops</code>.
	 *
	 * @param object {'from': string, 'to': string} структура,
	 * содержащая начальную и конечную точку маршрута.
	 */
	sort: function() {
		var numstops = this.points.from.length;

		this.defineStartAndFinish();
		console.log("\n* edges defined:\n\n");
		for (i in this.points.from) {
			console.log("from: " + this.points.from[i] + ", to: " + this.points.to[i]);
		} console.log("\n");

		// сортируем остальное
		for (var i = 0; i < numstops - 1; i++) {
			for (var j = i + 1; j < numstops; j++) {
				console.log("loop[" + i + "][" + j + "]");
				if (this.points.to[i] == this.points.from[j]) { // match
					if (j != i + 1) {
						// j == i+1 и равенство to[i] = from[j] означает,
						// что ничего менять не надо.
						//
						// свопнуть j с i+1 и break:
						this.swap(j, i+1);
						console.log("swap: " + (i+1) + "/" + j + ": " + this.points.to[i] + " with " + this.points.from[j]);
						break;
					} else {
						console.log("ok");
						// делать нечего
						break;
					}
				}

				if (j == numstops - 1) {
					this.swap(j, 0);
				}
			}
		}
	},

	/**
	 * Меняет местами i-ый элемент с j-ым в <code>this.points</code>.
	 *
	 * @param i int позиция элемента в <code>this.points</code>
	 * @param j int позиция элемента в <code>this.points</code>
	 */
	swap: function(i, j) {
		var tmp = {from: '', to: '', order: 0};
		tmp.from = this.points.from[i];
		tmp.to = this.points.to[i];
		tmp.order = this.points.orders[i];
		this.points.from[i] = this.points.from[j];
		this.points.to[i] = this.points.to[j];
		this.points.orders[i] = this.points.orders[j];
		this.points.from[j] = tmp.from;
		this.points.to[j] = tmp.to;
		this.points.orders[j] = tmp.order;
	},

	/**
	 * Устанавливает начальную и конечную точку маршрута, определяя
	 * их по уникальности (если конец не встречается в началах, то
	 * это пункт назначения; если начало не встречается в концах, то
	 * это пункт отправленя.
	 */
	defineStartAndFinish: function() {
		var n = this.points.from.length;
		for (var i = 0; i < n; i++) {
			for (var j = 0; j < n; j++) {
				if (i == j)
					continue;
				if (this.points.to[i] == this.points.from[j]) {
					break; // нашлось
				}
				if (j == n - 1) {
					this.swap(i, n - 1); // пункт назначения
				}
			}
		}

		for (var i = 0; i < n; i++) {
			for (var j = 0; j < n; j++) {
				if (i == j)
					continue;
				if (this.points.from[i] == this.points.to[j]) {
					break; // нашлось
				}
				if (j == n - 1) {
					this.swap(i, 0); // пункт отправления
				}
			}
		}
	},
	
	/**
	 * Выводит содержимое результат в конечный список.
	 *
	 * @param string target_id id конечного списка (<code><ol id="target_id">innerHTML</ol></code>)
	 */
	echoOrderedList: function(target_id) {
//		console.log(this.points);
//		console.log(this.stops);
		for (x in this.points.orders) {
			document.getElementById(target_id).innerHTML +=
					'<' + this.configs.itemTag + '>' + this.stops.texts[this.points.orders[x]] + '</' + this.configs.itemTag + '>';
		}
	}
}